package com.myoa.engineering.crawl.ppomppu.processor.infrastructure.client;

import com.myoa.engineering.crawl.ppomppu.processor.configuration.factory.WebFluxExchangeStragiesFactory;
import com.myoa.engineering.crawl.ppomppu.support.util.WebUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.stereotype.Component;
import org.springframework.web.reactive.function.client.WebClient;
import org.springframework.web.reactive.function.client.WebClientRequestException;
import reactor.core.publisher.Mono;
import reactor.core.scheduler.Schedulers;

/**
 * PpomppuBoardFeedRetriever
 *
 * @author Shin Woo-jin (woozu.shin@kakaoent.com)
 * @since 2021-09-08
 */
@Slf4j
@Component
public class PpomppuBoardFeedRetriever {

    private static final String PPOMPPU_URL = "https://www.ppomppu.co.kr/";

    private final WebClient webClient;

    public PpomppuBoardFeedRetriever(WebClient.Builder webClientBuilder) {
        this.webClient = webClientBuilder.baseUrl(PPOMPPU_URL)
                                         .exchangeStrategies(WebFluxExchangeStragiesFactory.getDefault())
                                         .defaultHeader("Content-Type", "text/html")
                                         .defaultHeader(WebUtil.HEADER_USER_AGENT_KEY, WebUtil.HEADER_USER_AGENT_VALUE)
                                         .build();
    }

    public Mono<String> getHtml(String uri) {
        return webClient.get()
                        .uri(uri)
                        .exchangeToMono(e -> e.bodyToMono(new ParameterizedTypeReference<String>() {}))
                        .publishOn(Schedulers.boundedElastic())
                        .onErrorResume(WebClientRequestException.class, t -> {
                            log.info("Exception occured, ignoring. : {}", t.getClass().getSimpleName());
                            return Mono.empty();
                        });
    }

}
